package com.zyzf.coffeepreorder.coffee.composeui

import android.annotation.SuppressLint
import android.content.res.Configuration
import android.graphics.Bitmap.CompressFormat
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.StrictMode
import android.os.StrictMode.ThreadPolicy
import android.util.Log
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.heightIn
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.outlined.Create
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.OutlinedIconButton
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableDoubleStateOf
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import coil.compose.AsyncImage
import coil.request.ImageRequest
import com.jcraft.jsch.Channel
import com.jcraft.jsch.ChannelSftp
import com.jcraft.jsch.JSch
import com.jcraft.jsch.JSchException
import com.jcraft.jsch.Session
import com.jcraft.jsch.SftpException
import com.zyzf.coffeepreorder.R
import com.zyzf.coffeepreorder.database.AppDatabase
import com.zyzf.coffeepreorder.database.model.Coffee
import com.zyzf.coffeepreorder.ui.theme.CoffeePreorderTheme
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.io.ByteArrayOutputStream
import java.io.File
import java.io.FileOutputStream
import java.util.Properties


@SuppressLint("UnrememberedMutableState")
@OptIn(ExperimentalMaterial3Api::class, ExperimentalFoundationApi::class,
    DelicateCoroutinesApi::class
)
@Composable
fun CoffeeList(navController: NavController?) {
    val openDialog = remember { mutableStateOf(false) }
    val add = remember { mutableStateOf(false) }
    val coffee = remember { mutableStateOf(Coffee("", 0.0, "", null, 0)) }
    val context = LocalContext.current
    val itemsList = remember { mutableStateListOf<Coffee>() }

    var imageUri: Any? by remember { mutableStateOf(R.drawable.img) }

    val photoPicker = rememberLauncherForActivityResult(
        contract = ActivityResultContracts.PickVisualMedia()
    ) {
        if (it != null) {
            Log.d("PhotoPicker", "Selected URI: $it")
            imageUri = it
        } else {
            Log.d("PhotoPicker", "No media selected")
        }
    }

    LaunchedEffect(Unit) {
        withContext(Dispatchers.IO) {
            AppDatabase.getInstance(context).coffeeDao().getAll().collect { data ->
                itemsList.clear()
                itemsList.addAll(data)
            }
        }
    }
    LazyColumn(
        horizontalAlignment = Alignment.CenterHorizontally) {
        items(itemsList) { currentCoffee ->
            Row(modifier = Modifier
                .fillMaxWidth()
                .heightIn(max = 140.dp)
                .padding(bottom = 10.dp, top = 10.dp),
                horizontalArrangement = Arrangement.SpaceAround) {

                AsyncImage(
                    model = ImageRequest.Builder(context = LocalContext.current).data("https://zyzf.space/s/zXgFRTmbR4KMxMH/download?path=&files=coffee_image_" + currentCoffee.uid +".png")
                        .crossfade(true).build(),
                    error = painterResource(R.drawable.ic_broken_image),
                    placeholder = painterResource(R.drawable.loading_img),
                    contentDescription = "Кофе",
                    contentScale = ContentScale.Crop,
                    modifier = Modifier
                        .size(100.dp)
                        .clip(RoundedCornerShape(50.dp))
                )

                Column(
                    Modifier
                        .weight(2f)
                        .padding(start = 20.dp)) {
                    Text(text = currentCoffee.name, fontSize = 25.sp)
                    Text(text = String.format("%.2f", currentCoffee.cost), fontSize = 20.sp)
                    Text(text = currentCoffee.ingredients, fontSize = 15.sp)
                    Row(
                        Modifier
                            .fillMaxWidth()
                            .padding(top = 5.dp)) {
                        Button(
                            onClick = {
                                GlobalScope.launch (Dispatchers.Main) {
                                    currentCoffee.uid?.let {
                                        AppDatabase.getInstance(context).cartDao().get().uid?.let { it1 ->
                                            AppDatabase.getInstance(context).cartDao().insertCoffee(
                                                it1,
                                                it,
                                                1
                                            )
                                        }
                                    }
                                    AppDatabase.getInstance(context).coffeeDao().getAll().collect { data ->
                                        itemsList.clear()
                                        itemsList.addAll(data)
                                    }
                                }
                            },
                            shape = CircleShape,
                            modifier = Modifier.fillMaxWidth(fraction = 0.75f)
                        ) {
                            Icon(
                                imageVector = Icons.Default.Add,
                                contentDescription = "Favorite",
                                modifier = Modifier.size(20.dp)
                            )
                            Spacer(Modifier.size(ButtonDefaults.IconSpacing))
                            Text(text = "В корзину")
                        }
                        OutlinedIconButton(
                            onClick = {
                                coffee.value = currentCoffee
                                add.value = false
                                openDialog.value = true
                            },
                            Modifier
                                .padding(start = 10.dp)
                                .clip(CircleShape)) {
                            Icon(
                                imageVector = Icons.Outlined.Create,
                                contentDescription = "Favorite",
                                modifier = Modifier.size(20.dp)
                            )
                        }
                    }
                }
            }
        }
    }
    Box(modifier = Modifier.fillMaxSize()) {
        FloatingActionButton(onClick = { coffee.value = Coffee("", 0.0, "", null, 0); add.value = true; openDialog.value = true },
            Modifier
                .padding(all = 20.dp)
                .align(alignment = Alignment.BottomEnd)) {
            Icon(
                imageVector = Icons.Filled.Add,
                contentDescription = "Add",
                modifier = Modifier.size(20.dp)
            )
        }
    }
    if (openDialog.value) {
        var name by remember { mutableStateOf(coffee.value.name) }
        var cost by remember { mutableDoubleStateOf(coffee.value.cost) }
        var ingredients by remember { mutableStateOf(coffee.value.ingredients) }

        ModalBottomSheet(onDismissRequest = { openDialog.value = false }) {
            Column(modifier = Modifier
                .fillMaxWidth()
                .padding(all = 10.dp),
                horizontalAlignment = Alignment.CenterHorizontally) {
                OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                    value = name, onValueChange = {name = it},
                    label = {
                        Text(stringResource(id = R.string.coffee_name))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text)
                )
                OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                    value = cost.toString(), onValueChange = {cost = it.toDouble()},
                    label = {
                        Text(stringResource(id = R.string.coffee_cost))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Decimal)
                )
                OutlinedTextField(modifier = Modifier.fillMaxWidth().padding(bottom = 10.dp),
                    value = ingredients, onValueChange = {ingredients = it},
                    label = {
                        Text(stringResource(id = R.string.coffee_ingredients))
                    },
                    keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text)
                )
                AsyncImage(
                    modifier = Modifier
                        .size(100.dp).clip(RoundedCornerShape(50.dp)).border(2.dp, MaterialTheme.colorScheme.outline, shape = RoundedCornerShape(50.dp))
                        .clickable {
                            photoPicker.launch(
                                PickVisualMediaRequest(
                                    ActivityResultContracts.PickVisualMedia.ImageOnly
                                )
                            )
                        }.border(2.dp, MaterialTheme.colorScheme.outlineVariant, shape = RoundedCornerShape(50.dp)),
                    contentDescription = "Кофе",
                    error = painterResource(R.drawable.ic_broken_image),
                    placeholder = painterResource(R.drawable.loading_img),
                    contentScale = ContentScale.Crop,
                    model = ImageRequest.Builder(LocalContext.current)
                        .data(if (coffee.value.name != "") "https://zyzf.space/s/zXgFRTmbR4KMxMH/download?path=&files=coffee_image_" + coffee.value.uid +".png" else imageUri)
                        .crossfade(enable = true)
                        .build(),
                )
                Row(modifier = Modifier.fillMaxWidth(), horizontalArrangement = Arrangement.Center) {
                    if (add.value) {
                        Button(onClick = {
                            GlobalScope.launch (Dispatchers.Main) {
                                val newCoffee: Long = AppDatabase.getInstance(context).coffeeDao().insert(
                                    name = name, cost = cost, ingredients = ingredients
                                )

                                val inputStream = context.contentResolver.openInputStream(imageUri as Uri)
                                val bitmap = BitmapFactory.decodeStream(inputStream)

                                val f = File(context.cacheDir, "coffee_image_$newCoffee.png")
                                withContext(Dispatchers.IO) {
                                    f.createNewFile()
                                    val bos = ByteArrayOutputStream()
                                    bitmap.compress(CompressFormat.PNG, 0 /*ignored for PNG*/, bos)
                                    val bitmapdata = bos.toByteArray()
                                    val fos = FileOutputStream(f)
                                    fos.write(bitmapdata)
                                    fos.flush()
                                    fos.close()
                                }

                                copyFileToSftp(f, "/mnt/nextcloud/data/Zyzf/files/Images")

                                AppDatabase.getInstance(context).coffeeDao().getAll().collect { data ->
                                    itemsList.clear()
                                    itemsList.addAll(data)
                                }
                                openDialog.value = false
                            }
                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Добавить")
                        }
                    } else {
                        Button(onClick = {
                            GlobalScope.launch (Dispatchers.Main) {
                                AppDatabase.getInstance(context).coffeeDao().update(
                                    Coffee(name = name, cost = cost, ingredients = ingredients, cartId = coffee.value.cartId, count = coffee.value.count)
                                )
                                AppDatabase.getInstance(context).coffeeDao().getAll().collect { data ->
                                    itemsList.clear()
                                    itemsList.addAll(data)
                                }
                            }

                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Изменить")
                        }
                        Spacer(modifier = Modifier.padding(all = 20.dp))
                        Button(onClick = {
                            GlobalScope.launch (Dispatchers.Main) {
                                AppDatabase.getInstance(context).coffeeDao().delete(
                                    coffee.value
                                )
                                AppDatabase.getInstance(context).coffeeDao().getAll().collect { data ->
                                    itemsList.clear()
                                    itemsList.addAll(data)
                                }
                                openDialog.value = false
                            }

                        }, modifier = Modifier.padding(0.dp, 10.dp, 0.dp, 30.dp)) {
                            Text("Удалить")
                        }
                    }
                }
            }
        }
    }
}

const val REMOTE_HOST = "109.197.199.134"
const val USERNAME = "zyzf"
const val PASSWORD = "250303Zyzf-d-grad"
const val REMOTE_PORT = 2223

fun copyFileToSftp(srcFile: File, ftpPath: String): Boolean {

    var jschSession: Session? = null

    try {
        val jsch = JSch()
        jsch.setKnownHosts("/home/mkyong/.ssh/known_hosts")
        jschSession = jsch.getSession(USERNAME, REMOTE_HOST, REMOTE_PORT)
        jschSession.setPassword(PASSWORD)

        val config = Properties()
        config["StrictHostKeyChecking"] = "no"
        jschSession.setConfig(config)

        val policy = ThreadPolicy.Builder().permitAll().build()
        StrictMode.setThreadPolicy(policy)
        jschSession.connect(10000)
        val sftp: Channel = jschSession.openChannel("sftp")

        sftp.connect(5000)
        val channelSftp: ChannelSftp = sftp as ChannelSftp

        channelSftp.put(srcFile.absolutePath, ftpPath)
        channelSftp.exit()

    } catch (e: JSchException) {
        e.printStackTrace()
        return false
    } catch (e: SftpException) {
        e.printStackTrace()
        return false
    } finally {
        jschSession?.disconnect()
    }
    return true
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun CoffeeListPreview() {
    CoffeePreorderTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            CoffeeList(navController = null)
        }
    }
}