package com.zyzf.coffeepreorder.coffee.composeui

import android.content.res.Configuration
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Button
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import com.zyzf.coffeepreorder.R
import com.zyzf.coffeepreorder.coffee.model.getCoffee
import com.zyzf.coffeepreorder.composeui.navigation.Screen
import com.zyzf.coffeepreorder.ui.theme.CoffeePreorderTheme

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CoffeeView(navController: NavController?, id: Int) {
    val coffee = getCoffee()[id]
    ModalBottomSheet(onDismissRequest = { navController?.navigate(Screen.CoffeeList.route) }) {
        Column(modifier = Modifier.fillMaxWidth().padding(all = 10.dp),
            horizontalAlignment = Alignment.CenterHorizontally) {
            OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                value = coffee.name, onValueChange = {}, readOnly = true,
                label = {
                    Text(stringResource(id = R.string.coffee_name))
                }
            )
            OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                value = coffee.cost.toString(), onValueChange = {}, readOnly = true,
                label = {
                    Text(stringResource(id = R.string.coffee_cost))
                }
            )
            OutlinedTextField(modifier = Modifier.fillMaxWidth(),
                value = coffee.ingredients, onValueChange = {}, readOnly = true,
                label = {
                    Text(stringResource(id = R.string.coffee_ingredients))
                }
            )
            Row(modifier = Modifier.fillMaxWidth()) {
                Button(onClick = { navController?.navigate(Screen.CoffeeView.route) }) {
                    Text("Изменить")
                }
            }
        }
    }
}

@Preview(name = "Light Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_NO)
@Preview(name = "Dark Mode", showBackground = true, uiMode = Configuration.UI_MODE_NIGHT_YES)
@Composable
fun StudentViewPreview() {
    CoffeePreorderTheme {
        Surface(
            color = MaterialTheme.colorScheme.background
        ) {
            CoffeeView(navController = null, id = 0)
        }
    }
}